@@ -1,3 +1,5 @@
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct C_original {
@@ -54,7 +56,7 @@ impl C_original {
         let default_compiler = String::from("gcc");
         self.compiler = default_compiler;
         if let Some(used_compiler) =
-            C_original::get_interpreter_option(&self.get_data(), "compiler")
+            C_original::get_interpreter_option(self.get_data(), "compiler")
         {
             if let Some(compiler_string) = used_compiler.as_str() {
                 info!("Using custom compiler: {}", compiler_string);
@@ -105,9 +107,11 @@ impl Interpreter for C_original {
     fn set_current_level(&mut self, level: SupportLevel) {
         self.support_level = level;
     }
-
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
 
     fn get_max_support_level() -> SupportLevel {
@@ -128,9 +132,9 @@ impl Interpreter for C_original {
             .replace(&[' ', '\t', '\n', '\r'][..], "")
             .is_empty()
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty() {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -234,19 +238,19 @@ impl Interpreter for C_original {
             .expect("Unable to start process");
         if output.status.success() {
             Ok(String::from_utf8(output.stdout).unwrap())
-        } else if C_original::error_truncate(&self.get_data()) == ErrTruncate::Short {
-            return Err(SniprunError::RuntimeError(
+        } else if C_original::error_truncate(self.get_data()) == ErrTruncate::Short {
+            Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr.clone())
                     .unwrap()
                     .lines()
                     .last()
                     .unwrap_or(&String::from_utf8(output.stderr).unwrap())
                     .to_owned(),
-            ));
+            ))
         } else {
-            return Err(SniprunError::RuntimeError(
+            Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr).unwrap(),
-            ));
+            ))
         }
     }
 }
