@@ -1,3 +1,6 @@
+#![allow(clippy::zombie_processes)]
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Clojure_fifo {
@@ -40,9 +43,9 @@ impl Clojure_fifo {
             pause = pause.saturating_add(std::time::Duration::from_millis(50));
 
             // timeout after 30s if no result found
-            if start.elapsed().as_secs() > 30 {
+            if start.elapsed().as_secs() > Clojure_fifo::get_repl_timeout(&self.data) {
                 return Err(SniprunError::InterpreterLimitationError(String::from(
-                    "reached the 30s timeout",
+                    "reached the repl timeout",
                 )));
             }
 
@@ -56,9 +59,9 @@ impl Clojure_fifo {
                     // info!("file : {:?}", contents);
                     if err_contents.contains(&end_mark_err) {
                         if let Some(index) = err_contents.rfind(&start_mark_err) {
-                            let mut err_to_display = err_contents[index + start_mark_err.len()
-                                ..err_contents.len() - end_mark_err.len() - 1]
-                                .to_owned();
+                            let index_e = err_contents.rfind(&end_mark_err).unwrap();
+                            let mut err_to_display =
+                                err_contents[index + start_mark_err.len()..index_e].to_owned();
                             info!("err to display : {:?}", err_to_display);
                             if !err_to_display.trim().is_empty() {
                                 info!("err found");
@@ -85,9 +88,8 @@ impl Clojure_fifo {
                     if out_contents.contains(&end_mark_ok) {
                         info!("out found");
                         let index = out_contents.rfind(&start_mark_ok).unwrap();
-                        return Ok(out_contents[index + start_mark_ok.len()
-                            ..out_contents.len() - end_mark_ok.len() - 1]
-                            .to_owned());
+                        let index_e = out_contents.rfind(&end_mark_ok).unwrap();
+                        return Ok(out_contents[index + start_mark_ok.len()..index_e].to_owned());
                     }
                 }
             }
@@ -103,7 +105,7 @@ impl Clojure_fifo {
         self.interpreter = default_interpreter;
         self.interpreter_repl = default_interpreter_repl;
         if let Some(used_interpreter) =
-            Clojure_fifo::get_interpreter_option(&self.get_data(), "interpreter")
+            Clojure_fifo::get_interpreter_option(self.get_data(), "interpreter")
         {
             if let Some(interpreter_string) = used_interpreter.as_str() {
                 info!("Using custom interpreter: {}", interpreter_string);
@@ -111,7 +113,7 @@ impl Clojure_fifo {
             }
         }
         if let Some(used_interpreter_repl) =
-            Clojure_fifo::get_interpreter_option(&self.get_data(), "interpreter_repl")
+            Clojure_fifo::get_interpreter_option(self.get_data(), "interpreter_repl")
         {
             if let Some(interpreter_string_repl) = used_interpreter_repl.as_str() {
                 info!("Using custom interpreter: {}", interpreter_string_repl);
@@ -177,10 +179,12 @@ impl Interpreter for Clojure_fifo {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Bloc
     }
@@ -194,11 +198,11 @@ impl Interpreter for Clojure_fifo {
             .is_empty()
             && self.get_current_level() >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.get_current_level() >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -213,7 +217,6 @@ impl Interpreter for Clojure_fifo {
         Ok(())
     }
     fn execute(&mut self) -> Result<String, SniprunError> {
-
         let output = Command::new(self.interpreter.split_whitespace().next().unwrap())
             .args(self.interpreter.split_whitespace().skip(1))
             .arg(&self.main_file_path)
@@ -222,7 +225,7 @@ impl Interpreter for Clojure_fifo {
             .expect("Unable to start process");
         if output.status.success() {
             Ok(String::from_utf8(output.stdout).unwrap())
-        } else if Clojure_fifo::error_truncate(&self.get_data()) == ErrTruncate::Short {
+        } else if Clojure_fifo::error_truncate(self.get_data()) == ErrTruncate::Short {
             Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr)
                     .unwrap()
@@ -293,13 +296,10 @@ impl ReplLikeInterpreter for Clojure_fifo {
             };
 
             self.save_code("kernel_launched\n".to_owned());
-
-            let pause = std::time::Duration::from_millis(300);
+            let pause = std::time::Duration::from_millis(100);
             std::thread::sleep(pause);
-
-            Err(SniprunError::CustomError(
-                "Clojure interactive kernel launched, re-run your snippet".to_owned(),
-            ))
+            let v = vec![(self.data.range[0] as usize, self.data.range[1] as usize)];
+            Err(SniprunError::ReRunRanges(v))
         }
     }
 
