@@ -1,3 +1,5 @@
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Generic {
@@ -169,7 +171,7 @@ impl Interpreter for Generic {
             Generic::generic_get_interpreter_option(&data, "compiler")
         {
             if let Some(compiler_string) = used_compiler.as_str() {
-                if ! compiler_string.is_empty() {
+                if !compiler_string.is_empty() {
                     info!("Using compiler: {}", compiler_string);
                     compiler = compiler_string.to_string();
                     config_name = fetched_config_name;
@@ -182,7 +184,7 @@ impl Interpreter for Generic {
             Generic::generic_get_interpreter_option(&data, "interpreter")
         {
             if let Some(interpreter_string) = used_interpreter.as_str() {
-                if ! interpreter_string.is_empty() {
+                if !interpreter_string.is_empty() {
                     info!("Using interpreter: {}", interpreter_string);
                     interpreter = interpreter_string.to_string();
                     config_name = fetched_config_name;
@@ -267,10 +269,12 @@ impl Interpreter for Generic {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Bloc
         //actually this has no importance since we're already in the 'fallback' generic interpreter
@@ -283,9 +287,9 @@ impl Interpreter for Generic {
             .replace(&[' ', '\t', '\n', '\r'][..], "")
             .is_empty()
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty() {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -328,7 +332,7 @@ impl Interpreter for Generic {
             );
             if output.status.success() {
                 return Ok(());
-            } else if Generic::error_truncate(&self.get_data()) == ErrTruncate::Short {
+            } else if Generic::error_truncate(self.get_data()) == ErrTruncate::Short {
                 return Err(SniprunError::CompilationError(
                     String::from_utf8(output.stderr.clone())
                         .unwrap()
@@ -368,7 +372,7 @@ impl Interpreter for Generic {
         );
         if output.status.success() {
             Ok(String::from_utf8(output.stdout).unwrap())
-        } else if Generic::error_truncate(&self.get_data()) == ErrTruncate::Short {
+        } else if Generic::error_truncate(self.get_data()) == ErrTruncate::Short {
             Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr.clone())
                     .unwrap()
