@@ -1,3 +1,6 @@
+#![allow(clippy::zombie_processes)]
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct JS_TS_bun {
@@ -29,9 +32,9 @@ impl JS_TS_bun {
             pause = pause.saturating_add(std::time::Duration::from_millis(50));
 
             // timeout after 30s if no result found
-            if start.elapsed().as_secs() > 30 {
+            if start.elapsed().as_secs() > JS_TS_bun::get_repl_timeout(&self.data) {
                 return Err(SniprunError::InterpreterLimitationError(String::from(
-                    "reached the 30s timeout",
+                    "reached the repl timeout",
                 )));
             }
 
@@ -64,7 +67,10 @@ impl JS_TS_bun {
                             } else {
                                 String::new()
                             };
-                        return Ok(relevant_content[index + start_mark.len()..index_end].to_owned() + "\n" + &possible_other);
+                        return Ok(relevant_content[index + start_mark.len()..index_end]
+                            .to_owned()
+                            + "\n"
+                            + &possible_other);
                     }
                 }
             }
@@ -123,10 +129,13 @@ impl Interpreter for JS_TS_bun {
     fn default_for_filetype() -> bool {
         false
     }
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
-    }
 
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
+    }
     fn get_max_support_level() -> SupportLevel {
         //define the max level support of the interpreter (see readme for definitions)
         SupportLevel::Bloc
@@ -155,11 +164,11 @@ impl Interpreter for JS_TS_bun {
             .is_empty()
             && self.support_level >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.support_level >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             // no code was retrieved
             self.code = String::from("");
@@ -209,13 +218,13 @@ impl Interpreter for JS_TS_bun {
             Ok(String::from_utf8(output.stdout).unwrap())
         } else {
             // return stderr
-            if JS_TS_bun::error_truncate(&self.get_data()) == ErrTruncate::Short {
+            if JS_TS_bun::error_truncate(self.get_data()) == ErrTruncate::Short {
                 Err(SniprunError::RuntimeError(
                     String::from_utf8(output.stderr.clone())
                         .unwrap()
                         .lines()
                         .filter(|l| l.contains("Error:"))
-                        .last()
+                        .next_back()
                         .unwrap_or(&String::from_utf8(output.stderr).unwrap())
                         .to_string(),
                 ))
