@@ -1,3 +1,6 @@
+#![allow(clippy::zombie_processes)]
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Julia_jupyter {
@@ -57,10 +60,12 @@ impl Interpreter for Julia_jupyter {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Bloc
     }
@@ -73,11 +78,11 @@ impl Interpreter for Julia_jupyter {
             .is_empty()
             && self.get_current_level() >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.get_current_level() >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -102,12 +107,12 @@ impl Interpreter for Julia_jupyter {
             .expect("Unable to start process");
         if output.status.success() {
             Ok(String::from_utf8(output.stdout).unwrap())
-        } else if Julia_jupyter::error_truncate(&self.get_data()) == ErrTruncate::Short {
+        } else if Julia_jupyter::error_truncate(self.get_data()) == ErrTruncate::Short {
             Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr.clone())
                     .unwrap()
                     .lines()
-                    .last()
+                    .next_back()
                     .unwrap_or(&String::from_utf8(output.stderr).unwrap())
                     .to_owned(),
             ))
@@ -191,15 +196,15 @@ impl ReplLikeInterpreter for Julia_jupyter {
         if String::from_utf8(output.stderr.clone()).unwrap().is_empty() {
             Ok(cleaned_result.join("\n") + "\n")
         } else {
-            return Err(SniprunError::RuntimeError(
+            Err(SniprunError::RuntimeError(
                 strip_ansi_escapes::strip_str(String::from_utf8_lossy(&output.stderr.clone()))
                     .lines()
                     .last()
-                    .unwrap_or(
-                        &strip_ansi_escapes::strip_str(String::from_utf8_lossy(&output.stderr))
-                    )
+                    .unwrap_or(&strip_ansi_escapes::strip_str(String::from_utf8_lossy(
+                        &output.stderr,
+                    )))
                     .to_owned(),
-            ));
+            ))
         }
     }
 }
