@@ -1,3 +1,6 @@
+#![allow(clippy::zombie_processes)]
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Julia_original {
@@ -47,14 +50,16 @@ impl Julia_original {
         }
 
         let index = contents.rfind(&start_mark).unwrap();
-        Ok(contents[index + start_mark.len()..contents.len() - end_mark.len() - 1].to_owned())
+
+        let index_e = contents.rfind(&end_mark).unwrap();
+        Ok(contents[index + start_mark.len()..index_e].to_owned())
     }
 
     fn fetch_config(&mut self) {
         let default_interpreter = String::from("julia");
         self.interpreter = default_interpreter;
         if let Some(used_interpreter) =
-            Julia_original::get_interpreter_option(&self.get_data(), "interpreter")
+            Julia_original::get_interpreter_option(self.get_data(), "interpreter")
         {
             if let Some(interpreter_string) = used_interpreter.as_str() {
                 info!("Using custom interpreter: {}", interpreter_string);
@@ -63,7 +68,7 @@ impl Julia_original {
         }
 
         if let Some(project_opt) =
-            Julia_original::get_interpreter_option(&self.get_data(), "project")
+            Julia_original::get_interpreter_option(self.get_data(), "project")
         {
             if let Some(mut project_str) = project_opt.as_str() {
                 if project_str == "." {
@@ -136,10 +141,12 @@ impl Interpreter for Julia_original {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Bloc
     }
@@ -159,11 +166,11 @@ impl Interpreter for Julia_original {
             .is_empty()
             && self.get_current_level() >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.get_current_level() >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -189,7 +196,7 @@ impl Interpreter for Julia_original {
             .expect("Unable to start process");
         if output.status.success() {
             Ok(String::from_utf8(output.stdout).unwrap())
-        } else if Julia_original::error_truncate(&self.get_data()) == ErrTruncate::Short {
+        } else if Julia_original::error_truncate(self.get_data()) == ErrTruncate::Short {
             Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr.clone())
                     .unwrap()
@@ -256,13 +263,11 @@ impl ReplLikeInterpreter for Julia_original {
                 ),
             };
 
+            self.save_code("kernel_launched\n".to_owned());
             let pause = std::time::Duration::from_millis(100);
             std::thread::sleep(pause);
-            self.save_code("kernel_launched".to_owned());
-
-            Err(SniprunError::CustomError(
-                "Julia kernel launched, re-run your snippet".to_owned(),
-            ))
+            let v = vec![(self.data.range[0] as usize, self.data.range[1] as usize)];
+            Err(SniprunError::ReRunRanges(v))
         }
     }
 
