@@ -1,3 +1,6 @@
+#![allow(clippy::zombie_processes)]
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Lua_nvim {
@@ -46,10 +49,12 @@ impl Interpreter for Lua_nvim {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Bloc
     }
@@ -66,10 +71,11 @@ impl Interpreter for Lua_nvim {
             //that doesn't work in nvim context for some reason
             // note that since Lua_original is the default and if lua_nvim is selected, we should
             // never take this code path
-            let mut good_interpreter = crate::interpreters::Lua_original::new_with_level(
-                self.data.clone(),
-                SupportLevel::Selected, //prevent fallback infinite loop
-            );
+            let mut good_interpreter =
+                crate::interpreters::Lua_original::Lua_original::new_with_level(
+                    self.data.clone(),
+                    SupportLevel::Selected, //prevent fallback infinite loop
+                );
             return Some(good_interpreter.run());
         }
         None
@@ -83,11 +89,11 @@ impl Interpreter for Lua_nvim {
             .is_empty()
             && self.get_current_level() >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.get_current_level() >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -95,6 +101,13 @@ impl Interpreter for Lua_nvim {
     }
 
     fn add_boilerplate(&mut self) -> Result<(), SniprunError> {
+        let code = self
+            .code
+            .lines()
+            .map(|line| line.strip_prefix("local ").unwrap_or(line))
+            .collect::<Vec<&str>>()
+            .join("\n");
+        self.code = code;
         Ok(())
     }
 
@@ -119,7 +132,7 @@ impl Interpreter for Lua_nvim {
         info!("yay from lua interpreter - in another nvim instance");
         if output.status.success() {
             Ok(String::from_utf8(output.stdout).unwrap())
-        } else if Lua_nvim::error_truncate(&self.get_data()) == ErrTruncate::Short {
+        } else if Lua_nvim::error_truncate(self.get_data()) == ErrTruncate::Short {
             Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr.clone())
                     .unwrap()
