@@ -1,3 +1,6 @@
+#![allow(clippy::zombie_processes)]
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Python3_fifo {
@@ -38,14 +41,18 @@ impl Python3_fifo {
             pause = pause.saturating_add(std::time::Duration::from_millis(50));
 
             // timeout after 30s if no result found
-            if start.elapsed().as_secs() > 30 {
-                return Err(SniprunError::InterpreterLimitationError(String::from("reached the 30s timeout")));
+            if start.elapsed().as_secs() > Python3_fifo::get_repl_timeout(&self.data) {
+                return Err(SniprunError::InterpreterLimitationError(String::from(
+                    "reached the repl timeout",
+                )));
             }
 
             // Python3_fifo-specific things to workaround nonblocking plot issues
             if start.elapsed().as_millis() > 150 {
                 let sync_repl_cmd = self.data.sniprun_root_dir.clone() + "/ressources/sync_repl.sh";
-                let res = Command::new(sync_repl_cmd).arg(self.cache_dir.clone()).output();
+                let res = Command::new(sync_repl_cmd)
+                    .arg(self.cache_dir.clone())
+                    .output();
                 info!(
                     "had to sync the repl because of timeout on awaiting result:\
                     happens  when a blocking command (plot, infinite loop) is run: {:?}",
@@ -63,9 +70,9 @@ impl Python3_fifo {
                     // info!("file : {:?}", contents);
                     if err_contents.contains(&end_mark) {
                         if let Some(index) = err_contents.rfind(&start_mark) {
-                            let mut err_to_display = err_contents
-                                [index + start_mark.len()..err_contents.len() - end_mark.len() - 1]
-                                .to_owned();
+                            let index_e = err_contents.rfind(&end_mark).unwrap();
+                            let mut err_to_display =
+                                err_contents[index + start_mark.len()..index_e].to_owned();
                             info!("err to display : {:?}", err_to_display);
                             if !err_to_display.trim().is_empty() {
                                 info!("err found");
@@ -94,9 +101,9 @@ impl Python3_fifo {
                     if out_contents.contains(&end_mark) {
                         info!("out found");
                         let index = out_contents.rfind(&start_mark).unwrap();
-                        return Ok(out_contents
-                            [index + start_mark.len()..out_contents.len() - end_mark.len() - 1]
-                            .to_owned());
+
+                        let index_e = out_contents.rfind(&end_mark).unwrap();
+                        return Ok(out_contents[index + start_mark.len()..index_e].to_owned());
                     }
                 }
             }
@@ -136,21 +143,35 @@ impl Python3_fifo {
             .replace(&[' ', '\t', '\n', '\r'][..], "")
             .is_empty()
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         }
-        for line in v.iter() {
+        let mut in_import_list = false;
+        for line in v.iter().filter(|line| !line.trim().starts_with('#')) {
             // info!("lines are : {}", line);
-            if (line.trim().starts_with("import ") || line.trim().starts_with("from"))  //basic selection
-                && !line.trim().starts_with('#')
-                    && self.module_used(line, &self.code)
+            if in_import_list {
+                self.imports = self.imports.clone() + "\n" + line;
+                if line.contains(')') {
+                    in_import_list = false;
+                }
+                continue;
+            }
+            if line.trim().starts_with("import ") || line.trim().starts_with("from")
+            //basic selection
             {
+                if line.contains('(') {
+                    self.imports = self.imports.clone() + "\n" + line;
+                    in_import_list = true;
+                    continue;
+                }
                 // embed in try catch blocs in case uneeded module is unavailable
 
-                let already_imported: String = self.read_previous_code();
-                if !already_imported.contains(line.trim()) {
-                    let line = line.trim_start();
-                    self.imports = self.imports.clone() + "\n" + line;
-                    self.save_code(already_imported + "\n" + line);
+                if self.module_used(line, &self.code) {
+                    let already_imported: String = self.read_previous_code();
+                    if !already_imported.contains(line.trim()) {
+                        let line = line.trim_start();
+                        self.imports = self.imports.clone() + "\n" + line;
+                        self.save_code(already_imported + "\n" + line);
+                    }
                 }
             }
         }
@@ -166,7 +187,7 @@ impl Python3_fifo {
             return true;
         }
         if line.contains(" as ") {
-            if let Some(name) = line.split(' ').last() {
+            if let Some(name) = line.replace(',', " ").split(' ').next_back() {
                 return code.contains(name);
             }
         }
@@ -206,7 +227,7 @@ impl Python3_fifo {
         let default_interpreter = String::from("python3");
         self.interpreter = default_interpreter;
         if let Some(used_interpreter) =
-            Python3_fifo::get_interpreter_option(&self.get_data(), "interpreter")
+            Python3_fifo::get_interpreter_option(self.get_data(), "interpreter")
         {
             if let Some(interpreter_string) = used_interpreter.as_str() {
                 info!("Using custom interpreter: {}", interpreter_string);
@@ -216,7 +237,7 @@ impl Python3_fifo {
 
         if let Ok(path) = env::current_dir() {
             if let Some(venv_array_config) =
-                Python3_fifo::get_interpreter_option(&self.get_data(), "venv")
+                Python3_fifo::get_interpreter_option(self.get_data(), "venv")
             {
                 if let Some(actual_vec_of_venv) = venv_array_config.as_array() {
                     for possible_venv in actual_vec_of_venv.iter() {
@@ -295,10 +316,12 @@ impl Interpreter for Python3_fifo {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Import
     }
@@ -313,11 +336,11 @@ impl Interpreter for Python3_fifo {
             .is_empty()
             && self.get_current_level() >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.get_current_level() >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -406,9 +429,7 @@ impl ReplLikeInterpreter for Python3_fifo {
                         .output()
                         .unwrap();
 
-                    return Err(SniprunError::CustomError(
-                        "python3 REPL exited".to_owned(),
-                    ));
+                    return Err(SniprunError::CustomError("python3 REPL exited".to_owned()));
                 }
                 Ok(Fork::Parent(_)) => {}
                 Err(_) => info!(
@@ -416,13 +437,11 @@ impl ReplLikeInterpreter for Python3_fifo {
                 ),
             };
 
+            self.save_code("kernel_launched\nimport sys".to_owned());
             let pause = std::time::Duration::from_millis(100);
             std::thread::sleep(pause);
-            self.save_code("kernel_launched\nimport sys".to_owned());
-
-            Err(SniprunError::CustomError(
-                "Python3 kernel launched, re-run your snippet".to_owned(),
-            ))
+            let v = vec![(self.data.range[0] as usize, self.data.range[1] as usize)];
+            Err(SniprunError::ReRunRanges(v))
         }
     }
 
@@ -455,7 +474,7 @@ impl ReplLikeInterpreter for Python3_fifo {
         let mut lines = vec![];
         for i in 0..(self.code.lines().count() - 1) {
             let l1 = self.code.lines().nth(i).unwrap();
-            let l2 = self.code.lines().nth(i+1).unwrap();
+            let l2 = self.code.lines().nth(i + 1).unwrap();
             let nw1 = l1.len() - l1.trim_start().len();
             let nw2 = l2.len() - l2.trim_start().len();
             lines.push(l1);
@@ -467,7 +486,6 @@ impl ReplLikeInterpreter for Python3_fifo {
 
         self.code = lines.join("\n");
 
-
         let mut run_ion = String::new();
         let mut run_ioff = String::new();
         if self.imports.contains("pyplot") {
