@@ -1,3 +1,6 @@
+#![allow(clippy::zombie_processes)]
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Python3_jupyter {
@@ -42,7 +45,7 @@ impl Python3_jupyter {
             .replace(&[' ', '\t', '\n', '\r'][..], "")
             .is_empty()
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         }
         for line in v.iter() {
             // info!("lines are : {}", line);
@@ -68,7 +71,7 @@ impl Python3_jupyter {
             return true;
         }
         if line.contains(" as ") {
-            if let Some(name) = line.split(' ').last() {
+            if let Some(name) = line.split(' ').next_back() {
                 return code.contains(name);
             }
         }
@@ -99,7 +102,9 @@ impl Python3_jupyter {
             if let Some(remaining) = timeout.checked_sub(step) {
                 timeout = remaining;
             } else {
-                return Err(SniprunError::CustomError(String::from("Timeout on jupyter kernel start expired")));
+                return Err(SniprunError::CustomError(String::from(
+                    "Timeout on jupyter kernel start expired",
+                )));
             }
         }
     }
@@ -159,10 +164,12 @@ impl Interpreter for Python3_jupyter {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Import
     }
@@ -176,11 +183,11 @@ impl Interpreter for Python3_jupyter {
             .is_empty()
             && self.get_current_level() >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.get_current_level() >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -215,7 +222,7 @@ impl Interpreter for Python3_jupyter {
             .expect("Unable to start process");
         if output.status.success() {
             Ok(String::from_utf8(output.stdout).unwrap())
-        } else if Python3_jupyter::error_truncate(&self.get_data()) == ErrTruncate::Short {
+        } else if Python3_jupyter::error_truncate(self.get_data()) == ErrTruncate::Short {
             Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr.clone())
                     .unwrap()
@@ -350,15 +357,15 @@ impl ReplLikeInterpreter for Python3_jupyter {
         if String::from_utf8(output.stderr.clone()).unwrap().is_empty() {
             Ok(cleaned_result.join("\n") + "\n")
         } else {
-            return Err(SniprunError::RuntimeError(
+            Err(SniprunError::RuntimeError(
                 strip_ansi_escapes::strip_str(String::from_utf8_lossy(&output.stderr.clone()))
                     .lines()
                     .last()
-                    .unwrap_or(
-                        &strip_ansi_escapes::strip_str(String::from_utf8_lossy(&output.stderr))
-                    )
+                    .unwrap_or(&strip_ansi_escapes::strip_str(String::from_utf8_lossy(
+                        &output.stderr,
+                    )))
                     .to_owned(),
-            ));
+            ))
         }
     }
 }
@@ -378,7 +385,7 @@ mod test_python3_jupyter {
 
         // should panic if not an Ok()
         let string_result = res.unwrap();
-        assert!(string_result.contains(&"a 1"));
+        assert!(string_result.contains("a 1"));
     }
 
     #[allow(dead_code)]
