@@ -1,3 +1,5 @@
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Python3_original {
@@ -43,17 +45,30 @@ impl Python3_original {
             .replace(&[' ', '\t', '\n', '\r'][..], "")
             .is_empty()
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         }
-        for line in v.iter() {
+        let mut in_import_list = false;
+        for line in v.iter().filter(|line| !line.trim().starts_with('#')) {
             // info!("lines are : {}", line);
-            if (line.trim().starts_with("import ") || line.trim().starts_with("from "))  //basic selection
-                && !line.trim().starts_with('#')
-            && self.module_used(line, &self.code)
-            {
-                // embed in try catch blocs in case uneeded module is unavailable
-                let line = unindent(line);
-                self.imports = self.imports.clone() + "\n" + &line;
+            if in_import_list {
+                self.imports = self.imports.clone() + "\n" + line;
+                if line.contains(')') {
+                    in_import_list = false;
+                }
+                continue;
+            }
+            if line.trim().starts_with("import ") || line.trim().starts_with("from ") {
+                //basic selection
+                if line.contains('(') {
+                    self.imports = self.imports.clone() + "\n" + line.trim();
+                    in_import_list = true;
+                    continue;
+                }
+                if self.module_used(line, &self.code) {
+                    // embed in try catch blocs in case uneeded module is unavailable
+                    let line = unindent(line);
+                    self.imports = self.imports.clone() + "\n" + line.trim();
+                }
             }
         }
         info!("import founds : {:?}", self.imports);
@@ -68,7 +83,7 @@ impl Python3_original {
             return true;
         }
         if line.contains(" as ") {
-            if let Some(name) = line.split(' ').last() {
+            if let Some(name) = line.split(' ').next_back() {
                 return code.contains(name);
             }
         }
@@ -89,7 +104,7 @@ impl Python3_original {
         let default_interpreter = String::from("python3");
         self.interpreter = default_interpreter;
         if let Some(used_interpreter) =
-            Python3_original::get_interpreter_option(&self.get_data(), "interpreter")
+            Python3_original::get_interpreter_option(self.get_data(), "interpreter")
         {
             if let Some(interpreter_string) = used_interpreter.as_str() {
                 info!("Using custom interpreter: {}", interpreter_string);
@@ -99,7 +114,7 @@ impl Python3_original {
 
         if let Ok(path) = env::current_dir() {
             if let Some(venv_array_config) =
-                Python3_original::get_interpreter_option(&self.get_data(), "venv")
+                Python3_original::get_interpreter_option(self.get_data(), "venv")
             {
                 if let Some(actual_vec_of_venv) = venv_array_config.as_array() {
                     for possible_venv in actual_vec_of_venv.iter() {
@@ -184,10 +199,12 @@ impl Interpreter for Python3_original {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Import
     }
@@ -202,11 +219,11 @@ impl Interpreter for Python3_original {
             .is_empty()
             && self.get_current_level() >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.get_current_level() >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -251,7 +268,7 @@ impl Interpreter for Python3_original {
             .expect("Unable to start process");
         if output.status.success() {
             Ok(String::from_utf8(output.stdout).unwrap())
-        } else if Python3_original::error_truncate(&self.get_data()) == ErrTruncate::Short {
+        } else if Python3_original::error_truncate(self.get_data()) == ErrTruncate::Short {
             Err(SniprunError::RuntimeError(
                 String::from_utf8(output.stderr.clone())
                     .unwrap()
@@ -311,7 +328,7 @@ impl ReplLikeInterpreter for Python3_original {
             final_code.push(')');
         }
 
-        self.code = final_code.clone();
+        self.code.clone_from(&final_code);
         // info!("---{}---", &final_code);
 
         Ok(())
