@@ -1,3 +1,6 @@
+#![allow(clippy::zombie_processes)]
+use crate::interpreters::import::*;
+
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
 pub struct Sage_fifo {
@@ -32,11 +35,17 @@ impl Sage_fifo {
         let mut err_contents = String::new();
 
         let mut pause = std::time::Duration::from_millis(50);
-        let _start = std::time::Instant::now();
+        let start = std::time::Instant::now();
         loop {
             std::thread::sleep(pause);
             pause = pause.saturating_add(std::time::Duration::from_millis(50));
 
+            if start.elapsed().as_secs() > Sage_fifo::get_repl_timeout(&self.data) {
+                return Err(SniprunError::InterpreterLimitationError(String::from(
+                    "reached the repl timeout",
+                )));
+            }
+
             //check for stderr first
             if let Ok(mut file) = std::fs::File::open(&err_path) {
                 info!("errfile exists");
@@ -47,9 +56,9 @@ impl Sage_fifo {
                     // info!("file : {:?}", contents);
                     if err_contents.contains(&end_mark) {
                         if let Some(index) = err_contents.rfind(&start_mark) {
-                            let err_to_display = err_contents
-                                [index + start_mark.len()..err_contents.len() - end_mark.len() - 1]
-                                .to_owned();
+                            let index_e = err_contents.rfind(&end_mark).unwrap();
+                            let err_to_display =
+                                err_contents[index + start_mark.len()..index_e].to_owned();
                             info!("err to display : {:?}", err_to_display);
                             if !err_to_display.trim().is_empty() {
                                 info!("err found");
@@ -104,7 +113,7 @@ impl Sage_fifo {
                         for try_error_indicator in error_indicators.iter() {
                             if out_contents_current.contains(try_error_indicator) {
                                 info!("stdout contains error indicator");
-                                err_contents = out_contents.clone();
+                                err_contents.clone_from(&out_contents);
                                 // info!("file : {:?}", contents);
                                 err_contents = err_contents.replace("sage: ", "");
                                 err_contents = err_contents.replace("---------------------------------------------------------------------------\n","");
@@ -165,7 +174,7 @@ impl Sage_fifo {
             .replace(&[' ', '\t', '\n', '\r'][..], "")
             .is_empty()
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         }
         for line in v.iter() {
             // info!("lines are : {}", line);
@@ -194,7 +203,7 @@ impl Sage_fifo {
             return true;
         }
         if line.contains(" as ") {
-            if let Some(name) = line.split(' ').last() {
+            if let Some(name) = line.split(' ').next_back() {
                 return code.contains(name);
             }
         }
@@ -216,7 +225,7 @@ impl Sage_fifo {
         let default_interpreter = String::from("sage");
         self.interpreter = default_interpreter;
         if let Some(used_interpreter) =
-            Sage_fifo::get_interpreter_option(&self.get_data(), "interpreter")
+            Sage_fifo::get_interpreter_option(self.get_data(), "interpreter")
         {
             if let Some(interpreter_string) = used_interpreter.as_str() {
                 info!("Using custom interpreter: {}", interpreter_string);
@@ -224,7 +233,7 @@ impl Sage_fifo {
             }
         }
         if let Some(user_sage_config) =
-            Sage_fifo::get_interpreter_option(&self.get_data(), "interpreter")
+            Sage_fifo::get_interpreter_option(self.get_data(), "interpreter")
         {
             if let Some(_user_sage_config_str) = user_sage_config.as_str() {
                 info!("Using user sage config");
@@ -287,14 +296,16 @@ impl Interpreter for Sage_fifo {
         self.support_level = level;
     }
 
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
-    }
-
     fn get_max_support_level() -> SupportLevel {
         SupportLevel::Import
     }
 
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
+    }
     fn default_for_filetype() -> bool {
         true
     }
@@ -309,11 +320,11 @@ impl Interpreter for Sage_fifo {
             .is_empty()
             && self.get_current_level() >= SupportLevel::Bloc
         {
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
         } else if !self.data.current_line.replace(' ', "").is_empty()
             && self.get_current_level() >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             self.code = String::from("");
         }
@@ -384,9 +395,7 @@ impl ReplLikeInterpreter for Sage_fifo {
                         .output()
                         .unwrap();
 
-                    return Err(SniprunError::CustomError(
-                        "sage REPL exited".to_owned(),
-                    ));
+                    return Err(SniprunError::CustomError("sage REPL exited".to_owned()));
                 }
                 Ok(Fork::Parent(_)) => {}
                 Err(_) => {
@@ -394,13 +403,11 @@ impl ReplLikeInterpreter for Sage_fifo {
                 }
             };
 
+            self.save_code("kernel_launched\n".to_owned());
             let pause = std::time::Duration::from_millis(100);
             std::thread::sleep(pause);
-            self.save_code("kernel_launched\n".to_string());
-
-            Err(SniprunError::CustomError(
-                "Sage kernel launched, re-run your snippet".to_owned(),
-            ))
+            let v = vec![(self.data.range[0] as usize, self.data.range[1] as usize)];
+            Err(SniprunError::ReRunRanges(v))
         }
     }
 
