@@ -1,4 +1,5 @@
 // Be sure to read the CONTRIBUTING.md file :-)
+use crate::interpreters::import::*;
 
 #[derive(Clone)]
 #[allow(non_camel_case_types)]
@@ -66,9 +67,11 @@ impl Interpreter for Language_subname {
     fn set_current_level(&mut self, level: SupportLevel) {
         self.support_level = level;
     }
-
-    fn get_data(&self) -> DataHolder {
-        self.data.clone()
+    fn get_data_mut(&mut self) -> &mut DataHolder {
+        &mut self.data
+    }
+    fn get_data(&self) -> &DataHolder {
+        &self.data
     }
 
     fn get_max_support_level() -> SupportLevel {
@@ -93,13 +96,13 @@ impl Interpreter for Language_subname {
         {
             // if bloc is not pseudo empty and has Bloc current support level,
             // add fetched code to self
-            self.code = self.data.current_bloc.clone();
+            self.code.clone_from(&self.data.current_bloc);
 
         // if there is only data on current line / or Line is the max support level
         } else if !self.data.current_line.replace(" ", "").is_empty()
             && self.support_level >= SupportLevel::Line
         {
-            self.code = self.data.current_line.clone();
+            self.code.clone_from(&self.data.current_line);
         } else {
             // no code was retrieved
             self.code = String::from("");
@@ -124,7 +127,6 @@ impl Interpreter for Language_subname {
         write(&self.main_file_path, &self.code)
             .expect("Unable to write to file for language_subname");
 
-
         //fetch the option from the configuration
         //  interpreter_options = {
         //   example_original = {
@@ -140,8 +142,6 @@ impl Interpreter for Language_subname {
             }
         }
 
-
-
         let compiler = Language_subname::get_compiler_or(&self.data, "compiler");
         //compile it (to the bin_path that already points to the rigth path)
         let output = Command::new(compiler)
