@@ -14,6 +14,7 @@ use std::thread;
 pub mod daemonizer;
 pub mod display;
 pub mod error;
+pub mod input;
 pub mod interpreter;
 pub mod interpreters;
 pub mod launcher;
@@ -72,6 +73,9 @@ pub struct DataHolder {
     pub cli_args: Vec<String>,
 
     pub nvim_pid: usize,
+
+    // getting this sometimes fail, so better save it in sniprun's memory
+    pub sniprun_namespace_id_cache: Option<i64>,
 }
 
 #[derive(Clone, Default, Debug)]
@@ -122,6 +126,7 @@ impl Default for DataHolder {
             display_no_output: vec![DisplayType::Classic(Both)],
             cli_args: vec![],
             nvim_pid: 0,
+            sniprun_namespace_id_cache: None,
         }
     }
 }
@@ -270,7 +275,7 @@ impl EventHandler {
             //get filetype
             let ft = self.nvim.lock().unwrap().command_output("set ft?");
             if let Ok(real_ft) = ft {
-                self.data.filetype = String::from(real_ft.split('=').last().unwrap());
+                self.data.filetype = String::from(real_ft.split('=').next_back().unwrap());
             }
             info!("[FILLDATA] got filetype");
         }
@@ -448,7 +453,7 @@ enum HandleAction {
 pub fn start() {
     let mut event_handler = EventHandler::new();
     let _ = log_to_file(
-        &format!("{}/{}", event_handler.data.work_dir, "sniprun.log"),
+        format!("{}/{}", event_handler.data.work_dir, "sniprun.log"),
         LevelFilter::Info,
     );
     log_panics::init();
@@ -518,11 +523,11 @@ pub fn start() {
                             let result = launcher.select_and_run();
                             info!("[RUN] Interpreter return a result");
                             data.range[1] += 1; // display on end of code bloc
-                            display(result, nvim, &data);
+                            display(result, nvim, &mut data);
                         }
                     } else {
                         // normal, unique result
-                        display(result, event_handler2.nvim, &event_handler2.data);
+                        display(result, event_handler2.nvim, &mut event_handler2.data);
                     }
 
                     //clean data
@@ -579,7 +584,7 @@ mod test_main {
     #[test]
     fn test_main() {
         let mut event_handler = fake_event();
-        let _ = log_to_file(&"test_sniprun.log".to_string(), LevelFilter::Info);
+        let _ = log_to_file("test_sniprun.log", LevelFilter::Info);
 
         event_handler.fill_data(&fake_msgpack());
         event_handler.data.filetype = String::from("javascript");
@@ -590,7 +595,7 @@ mod test_main {
         let result = launcher.select_and_run();
         info!("[RUN] Interpreter return a result");
 
-        display(result, event_handler.nvim, &event_handler.data);
+        display(result, event_handler.nvim, &mut event_handler.data);
     }
 
     pub fn fake_event() -> EventHandler {
